<?php
# MetInfo Enterprise Content Management System 
# Copyright (C) MetInfo Co.,Ltd (http://www.metinfo.cn). All rights reserved. 
defined('IN_MET') or exit('No permission');
ini_set('date.timezone','Asia/Shanghai');
load::mod_class('pay/web/pay');
class tenpay extends pay {
    public $tem_config;
    public function __construct() {
        global $_M;
        parent::__construct();
        $this->tem_config['partner'] = $this->GetAPI('2', 'tem_partner');
        $this->tem_config['key']     = $this->GetAPI('2', 'tem_key');
    }
    /**
     * 财付通
     * @param String(32)  body              商品描述（商品或支付单简要描述）
     * @param String(127) attach            附加数据（在查询API和支付通知中原样返回，该字段主要用于商户携带订单的自定义数据）
     * @param String(32)  out_trade_no      商户订单号（商户支付的订单号由商户自定义生成，长度不大于32位、可包含字母）
     * @param Int         total_fee         总金额（订单总金额，原始单位为分，只能为整数。可后台处理为“元”为单位）
     * @param String(32)  goods_tag         商品标记（商品标记，代金券或立减优惠功能的参数）
     * @param String(32)  product_id        商品ID（trade_type=NATIVE，此参数必传。此id为二维码中包含的商品ID，商户自行定义）
     * @param String(255) return_url        返回URL（交易完成后跳转的URL，需给绝对路径，255字符内，格式如:http://wap.tenpay.com/ tenpay.asp，通过该路径直接将支付结果以Get的方式返回）
     * @param String(255) notify_url        通知URL（接收财付通通知的URL，需给绝对路径，255字符内，格式如:http://wap.tenpay.com/ tenpay.asp）
     * @param String(15)  spbill_create_ip  用户IP（订单生成的机器IP，指用户浏览器端IP，不是商户服务器IP）
     */
    public function tenpay($date) {
        global $_M;
        require_once ("tenpay/classes/RequestHandler.class.php");
        $notify_url = is_strinclude($_M['url']['pay_notify'], 'localhost')?'http://mall.kisbox.com/pay/notify_url.php':$_M['url']['pay_notify'];
        $return_url = is_strinclude($_M['url']['pay_return'], 'localhost')?'http://mall.kisbox.com/pay/return_url.php':$_M['url']['pay_return'];
        
        /* 创建支付请求对象 */
        $reqHandler = new RequestHandler();
        $reqHandler->init();
        $reqHandler->setKey(trim($this->tem_config['key']));
        $reqHandler->setGateUrl("https://gw.tenpay.com/gateway/pay.htm");
        //----------------------------------------
        //设置支付参数 
        //----------------------------------------
        $reqHandler->setParameter("partner", trim($this->tem_config['partner']));                     //商户号
        $reqHandler->setParameter("out_trade_no", $date['out_trade_no']);   //商户订单号
        $reqHandler->setParameter("total_fee", $date['total_fee']*100);     //总金额，*金额单位为分*
        $reqHandler->setParameter("return_url", $return_url);               //返回URL
        $reqHandler->setParameter("notify_url", $notify_url);               //通知URL
        $reqHandler->setParameter("body", $date['body']);                   //商品描述
        $reqHandler->setParameter("bank_type", "DEFAULT");                  //银行类型，默认为财付通
        //用户ip
        $reqHandler->setParameter("spbill_create_ip", $_SERVER['REMOTE_ADDR']); //客户端IP
        $reqHandler->setParameter("fee_type", "1");                             //币种
        $reqHandler->setParameter("subject",$date['subject']);                  //商品名称，（中介交易时必填）
        //系统可选参数
        $reqHandler->setParameter("sign_type", "MD5");            //签名方式，默认为MD5，可选RSA
        $reqHandler->setParameter("service_version", "1.0"); 	  //接口版本号
        $reqHandler->setParameter("input_charset", "utf-8");   	  //字符集
        $reqHandler->setParameter("sign_key_index", "1");    	  //密钥序号
        //业务可选参数
        $reqHandler->setParameter("attach", "");             	  //附件数据，原样返回就可以了
        $reqHandler->setParameter("product_fee", "");        	  //商品费用
        $reqHandler->setParameter("transport_fee", "0");      	  //物流费用
        $reqHandler->setParameter("time_start", date("YmdHis"));  //订单生成时间
        $reqHandler->setParameter("time_expire", "");             //订单失效时间
        $reqHandler->setParameter("buyer_id", "");                //买方财付通帐号
        $reqHandler->setParameter("goods_tag", "");               //商品标记
        $reqHandler->setParameter("trade_mode","1");              //交易模式（1.即时到帐模式，2.中介担保模式，3.后台选择（卖家进入支付中心列表选择））
        $reqHandler->setParameter("transport_desc","");           //物流说明
        $reqHandler->setParameter("trans_type","1");              //交易类型
        $reqHandler->setParameter("agentid","");                  //平台ID
        $reqHandler->setParameter("agent_type","");               //代理模式（0.无代理，1.表示卡易售模式，2.表示网店模式）
        $reqHandler->setParameter("seller_id","");                //卖家的商户号
        //请求的URL
        $reqUrl = $reqHandler->getRequestURL();
        $reqHandler->getDebugInfo();
        header("Location:{$reqUrl}");  
    }
    
    /**
     * 异步通知处理
     */
    public function donotify($date) {
        global $_M;
        require_once ("tenpay/classes/ResponseHandler.class.php");
        require_once ("tenpay/classes/RequestHandler.class.php");
        require_once ("tenpay/classes/client/ClientResponseHandler.class.php");
        require_once ("tenpay/classes/client/TenpayHttpClient.class.php");

	/* 创建支付应答对象 */
        $resHandler = new ResponseHandler();
        $resHandler->setKey(trim($this->tem_config['key']));
	//判断签名
        if($resHandler->isTenpaySign()) {
            //通知id
            $notify_id = $resHandler->getParameter("notify_id");
            //通过通知ID查询，确保通知来至财付通
            //创建查询请求
            $queryReq = new RequestHandler();
            $queryReq->init();
            $queryReq->setKey(trim($this->tem_config['key']));
            $queryReq->setGateUrl("https://gw.tenpay.com/gateway/simpleverifynotifyid.xml");
            $queryReq->setParameter("partner", trim($this->tem_config['partner']));                     //商户号
            $queryReq->setParameter("notify_id", $notify_id);
            //通信对象
            $httpClient = new TenpayHttpClient();
            $httpClient->setTimeOut(5);
            //设置请求内容
            $httpClient->setReqContent($queryReq->getRequestURL());
            //后台调用
            if($httpClient->call()) {
                //设置结果参数
                $queryRes = new ClientResponseHandler();
                $queryRes->setContent($httpClient->getResContent());
                $queryRes->setKey(trim($this->tem_config['key']));
                if($resHandler->getParameter("trade_mode") == "1"){
                    //判断签名及结果（即时到帐）
                    //只有签名正确,retcode为0，trade_state为0才是支付成功
                    if($queryRes->isTenpaySign() && $queryRes->getParameter("retcode") == "0" && $resHandler->getParameter("trade_state") == "0") {
                        //取结果参数做业务处理
                        $out_trade_no = $resHandler->getParameter("out_trade_no");
                        //财付通订单号
                        $transaction_id = $resHandler->getParameter("transaction_id");
                        //金额,以分为单位
                        $total_fee = $resHandler->getParameter("total_fee");
                        //如果有使用折扣券，discount有值，total_fee+discount=原请求的total_fee
                        $discount = $resHandler->getParameter("discount");
                        return TRUE;
                    } else {
                        return FALSE;
                    }
		} else if ($resHandler->getParameter("trade_mode") == "2") {
                    //判断签名及结果（中介担保）
                    //只有签名正确,retcode为0，trade_state为0才是支付成功
                    if($queryRes->isTenpaySign() && $queryRes->getParameter("retcode") == "0" ) {
                        //取结果参数做业务处理
			$out_trade_no = $resHandler->getParameter("out_trade_no");
                        //财付通订单号
			$transaction_id = $resHandler->getParameter("transaction_id");
			switch ($resHandler->getParameter("trade_state")) {
                            case "0":	//付款成功
                                    break;
                            case "1":	//交易创建
                                    break;
                            case "2":	//收获地址填写完毕
                                    break;
                            case "4":	//卖家发货成功
                                    break;
                            case "5":	//买家收货确认，交易成功
                                    break;
                            case "6":	//交易关闭，未完成超时关闭
                                    break;
                            case "7":	//修改交易价格成功
                                    break;
                            case "8":	//买家发起退款
                                    break;
                            case "9":	//退款成功
                                    break;
                            case "10":	//退款关闭		
                                    break;
                            default:
                                    break;
			}
			return TRUE;
                    } else {
                        return FALSE;
                    }
		}
            }else {
                return FALSE;
            } 
        } else  {
            return FALSE;
        }
    }
    
    /**
     * 同步通知处理
     */
    public function doreturn($date) {
        global $_M;
        require_once ("tenpay/classes/ResponseHandler.class.php");
//        require_once ("tenpay/classes/function.php");                           //日志记录
        
        /* 创建支付应答对象 */
        $resHandler = new ResponseHandler();
        $resHandler->setKey(trim($this->tem_config['key']));
        //判断签名
        if($resHandler->isTenpaySign()) {
            //通知id
            $notify_id = $resHandler->getParameter("notify_id");
            //商户订单号
            $out_trade_no = $resHandler->getParameter("out_trade_no");
            //财付通订单号
            $transaction_id = $resHandler->getParameter("transaction_id");
            //金额,以分为单位
            $total_fee = $resHandler->getParameter("total_fee");
            //如果有使用折扣券，discount有值，total_fee+discount=原请求的total_fee
            $discount = $resHandler->getParameter("discount");
            //支付结果
            $trade_state = $resHandler->getParameter("trade_state");
            //交易模式,1即时到账
            $trade_mode = $resHandler->getParameter("trade_mode");
            if("1" == $trade_mode ) {
                if("0" == $trade_state){ 
                    return TRUE;
                } else {
                    return FALSE;
                }
            }else if( "2" == $trade_mode  ) {
                if( "0" == $trade_state) {
                    return TRUE;
                } else {
                    return FALSE;
                }
            }
        } else {
            return FALSE;
        }
    }
}
# This program is an open source system, commercial use, please consciously to purchase commercial license.
# Copyright (C) MetInfo Co., Ltd. (http://www.metinfo.cn). All rights reserved.
?>